/*******************************************************************************
    Copyright (c) 2011 Yahoo! Inc. All rights reserved.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License. See accompanying LICENSE file.

    The Initial Developer of the Original Code is Shravan Narayanamurthy.
******************************************************************************/
/*
 * MedLDA_Test_Data_Formatter.cpp
 *
 *  Created on: 28-Jan-2011
 *      
 */
#include <mpi.h>

#include "MedLDA_Test_Data_Formatter.h"
#include "commons/Context.h"

#include <time.h>

namespace medlda {

MedLDA_Test_Data_Formatter::MedLDA_Test_Data_Formatter() {
    Context& context = Context::get_instance();
    string dumpfile = context.get_string("dumpfile");
    LOG(WARNING) << "Initializing Dictionary from " << dumpfile;
    //_global_dict.initialize_from_dump(dumpfile);
    //LOG(WARNING) << "Num of unique words: " << _global_dict.get_num_words();
    _dict.initialize_from_dump(dumpfile);
    noov = 0;
}

MedLDA_Test_Data_Formatter::~MedLDA_Test_Data_Formatter() {
}

void MedLDA_Test_Data_Formatter::format() {
    Context& context = Context::get_instance();
    string fname = context.get_string("corpus");
    int num_label = context.get_int("num_label");

    ifstream fin(fname.c_str());
    if (!fin) {
    	printf("ERROR> open %s failed!\n", fname.c_str());	
    	exit(1);
    }
    int world_rank;
    MPI_Comm_rank(MPI_COMM_WORLD, &world_rank);
    stringstream ss;
    ss << "." << world_rank;
    string dotrank = ss.str();
    string gndprefix = context.get_string("gndprefix");
    string gndlabel = gndprefix + ".label" + dotrank;
    string gndcount = gndprefix + ".count" + dotrank;
    ofstream gnd_fout(gndlabel.c_str());
    if (!gnd_fout) {
    	printf("ERROR> open %s failed!\n", gndlabel.c_str());	
    	exit(1);
    }
    ofstream gnd_count_fout(gndcount.c_str());
    if (!gnd_count_fout) {
    	printf("ERROR> open %s failed!\n", gndcount.c_str());	
    	exit(1);
    }
    std::string buffer_line;
    std::vector<int> label_vector;
    std::vector<string> word_vector;
    while(std::getline(fin, buffer_line)) {
      LDA::MedLDA_document wdoc;
      wdoc.set_docid(_num_docs++);

      bool jumped = false;
      label_vector.clear();
      word_vector.clear();
      size_t start_pos = 0;
      size_t first_colon = buffer_line.find(":");
      while(1) {
        size_t b_pos = buffer_line.find(" ", start_pos);
        if (b_pos == std::string::npos) {
          size_t colon = buffer_line.find(":", start_pos);
          if (colon == std::string::npos) {
            printf("error line: %s\n", buffer_line.c_str());
            exit(1);
          }
          int count = atoi(&buffer_line[colon+1]);
          string word_id = buffer_line.substr(start_pos, colon-start_pos);
          for (int i = 0; i < count; ++i) {
            word_vector.push_back(word_id);
          }
          break;
        }
        if (b_pos < first_colon) {
          // label
            int label_id = atoi(&buffer_line[start_pos]);
            if (label_id < num_label) {
                label_vector.push_back(label_id);
            }
        } else {
          // word
          size_t colon = buffer_line.find(":", start_pos);
          if (colon == std::string::npos) {
            printf("error line: %s\n", buffer_line.c_str());
            exit(1);
          }
          int count = atoi(&buffer_line[colon+1]);
          string word_id = buffer_line.substr(start_pos, colon-start_pos);
          for (int i = 0; i < count; ++i) {
            word_vector.push_back(word_id);
          }
        }
        start_pos = b_pos + 1;
      }
      for (size_t i = 0; i < label_vector.size(); ++i) {
          wdoc.add_gnd_label(label_vector[i]);
          if (i != 0) {
            gnd_fout << ",";
          }
          gnd_fout << label_vector[i];
      }
      gnd_fout << "\n";
      gnd_count_fout << label_vector.size() << "\n";
      int idx;
      for (size_t i = 0; i < word_vector.size(); ++i) {
          idx = _dict.get_index(word_vector[i]);
          if (idx == -1) {
              ++noov;
              idx = rand() % _dict.get_num_words();
          }
          wdoc.add_body(idx);
      }
        int num_words_cur_doc = wdoc.body_size();
        if (num_words_cur_doc > 0) {
            _num_words_in_all_docs += num_words_cur_doc;
            _doc_writer->write(wdoc);
        }
    }
    printf("---- Num of OOV: %d\n", noov);
}

}
