/*******************************************************************************
    Copyright (c) 2011 Yahoo! Inc. All rights reserved.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License. See accompanying LICENSE file.

    The Initial Developer of the Original Code is Shravan Narayanamurthy.
******************************************************************************/
/*
 * MedLDA_Train_Data_Formatter.cpp
 *
 *  Created on: 11-Jan-2011
 *      
 */

#include "MedLDA_Train_Data_Formatter.h"
#include "glog/logging.h"
#include "commons/Context.h"
#include "commons/document.pb.h"
#include "commons/constants.h"

#include <stdlib.h>
#include <stdio.h>
#include <vector>

namespace medlda {


MedLDA_Train_Data_Formatter::MedLDA_Train_Data_Formatter() {
    _num_docs = 0;
    _num_words_in_all_docs = 0;
    //using namespace boost;
    //Read the stop words into a set
    //stringstream ss;
    //for (int i = 0; i < NUM_STP_WRDS; i++) {
    //    ss << stopwords[i] << " : ";
    //    _stopWords.insert(stopwords[i]);
    //}
    //LOG(INFO) << ss.str();
    Context& context = Context::get_instance();
    string output_w = context.get_string("output_w");
    if (output_w != "")
        _doc_writer = new DocumentWriter(output_w);
}

MedLDA_Train_Data_Formatter::~MedLDA_Train_Data_Formatter() {
    delete _doc_writer;
}

void MedLDA_Train_Data_Formatter::format() {
    string fname = Context::get_instance().get_string("corpus");
    int num_label = Context::get_instance().get_int("num_label");

    ifstream fin(fname.c_str());
    if (!fin) {
    	printf("ERROR> open %s failed!\n", fname.c_str());	
    	exit(1);
    }
    std::string buffer_line;
    std::vector<int> label_vector;
    std::vector<string> word_vector;
    while(std::getline(fin, buffer_line)) {
      LDA::MedLDA_document wdoc;
      wdoc.set_docid(_num_docs++);

      bool jumped = false;
      label_vector.clear();
      word_vector.clear();
      size_t start_pos = 0;
      size_t first_colon = buffer_line.find(":");
      while(1) {
        size_t b_pos = buffer_line.find(" ", start_pos);
        if (b_pos == std::string::npos) {
          size_t colon = buffer_line.find(":", start_pos);
          if (colon == std::string::npos) {
            printf("error line: %s\n", buffer_line.c_str());
            exit(1);
          }
          int count = atoi(&buffer_line[colon+1]);
          string word_id = buffer_line.substr(start_pos, colon-start_pos);
          for (int i = 0; i < count; ++i) {
            word_vector.push_back(word_id);
          }
          break;
        }
        if (b_pos < first_colon) {
          // label
            int label_id = atoi(&buffer_line[start_pos]);
            if (label_id < num_label) {
                label_vector.push_back(label_id);
            }
        } else {
          // word
          size_t colon = buffer_line.find(":", start_pos);
          if (colon == std::string::npos) {
            printf("error line: %s\n", buffer_line.c_str());
            exit(1);
          }
          int count = atoi(&buffer_line[colon+1]);
          string word_id = buffer_line.substr(start_pos, colon-start_pos);
          for (int i = 0; i < count; ++i) {
            word_vector.push_back(word_id);
          }
        }
        start_pos = b_pos + 1;
      }
      for (size_t i = 0; i < label_vector.size(); ++i) {
          wdoc.add_gnd_label(label_vector[i]);
      }
      int idx;
      for (size_t i = 0; i < word_vector.size(); ++i) {
          idx = _dict.insert_word(word_vector[i]);
          wdoc.add_body(idx);
      }
        int num_words_cur_doc = wdoc.body_size();
        if (num_words_cur_doc > 0) {
            _num_words_in_all_docs += num_words_cur_doc;
            _doc_writer->write(wdoc);
        }
    }
    printf("----- num_terms: %d\n", _dict.get_num_words());
}

int MedLDA_Train_Data_Formatter::get_num_docs() {
    return _num_docs;
}

int MedLDA_Train_Data_Formatter::get_total_num_words() {
    return _num_words_in_all_docs;
}

WordIndexDictionary& MedLDA_Train_Data_Formatter::get_dictionary() {
    return _dict;
}

}
