/*******************************************************************************
    Copyright (c) 2011 Yahoo! Inc. All rights reserved.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License. See accompanying LICENSE file.

    The Initial Developer of the Original Code is Shravan Narayanamurthy.
******************************************************************************/
/*
 * MedLDA_Model.h
 *
 *  Created on: 05-Jan-2011
 *      
 */

#ifndef MEDLDA_MODEL_H_
#define MEDLDA_MODEL_H_

#include "TopicLearner/Model.h"
#include <string>
#include "TopicLearner/Parameter.h"
#include "TypeTopicCounts.h"
#include "TopicLearner/GenericTopKList.h"

namespace medlda {

using namespace std;

class MedLDA_Model: public Model {
public:
    const static int ALPHA = 1;
    const static int BETA = 2;
    const static int ETA = 3;
public:
    MedLDA_Model(int, int);
    virtual ~MedLDA_Model();

    Parameter& get_parameter(int);
    void set_parameter(int, Parameter&);
    TypeTopicCounts& get_ttc();

    Parameter* get_eta();
    void set_eta(Parameter*);

    double get_eval();

    bool save();

    void write_statistics(WordIndexDictionary&);

    double* init_max_score(size_t);
    int* init_prediction(size_t);
    void judge_multic(int, int);
    int get_multic_acc();

private:
    TypeTopicCounts* _ttc;
    param _alpha, _beta; 
    param *_eta;
    typedef GenericTopKList<wppair, wppair_gt> topK_word_prop_t;
    topK_word_prop_t** top_words_per_topic;
    bool _top_words_empty;

    double *max_score;
    int *pred_label;
    int multic_acc;
};

}
#endif /* MEDLDA_MODEL_H_ */
