/*******************************************************************************
    Copyright (c) 2011 Yahoo! Inc. All rights reserved.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License. See accompanying LICENSE file.

    The Initial Developer of the Original Code is Shravan Narayanamurthy.
******************************************************************************/
/*
 * MedLDA_Model_Tester.h
 *
 *  Created on: 06-Jan-2011
 *      
 */

#ifndef MEDLDA_MODEL_TESTER_H_
#define MEDLDA_MODEL_TESTER_H_
#include <tbb/mutex.h>

#include "TopicLearner/Model_Refiner.h"
#include "TypeTopicCounts.h"
#include "TopicLearner/Parameter.h"
#include "DocumentReader.h"
#include "DocumentWriter.h"
#include <boost/random/variate_generator.hpp>
#include <boost/random/uniform_real.hpp>
#include "WordIndexDictionary.h"

#include <vector>

namespace medlda {

using namespace boost;
using namespace std;

typedef struct {
    int label;
    double score;
} label_score_t;

class MedLDA_Model_Tester: public Model_Refiner {
public:
    MedLDA_Model_Tester(TypeTopicCounts&, Parameter&, Parameter&, Parameter*, size_t, 
            bool no_init = false);
    virtual ~MedLDA_Model_Tester();

    google::protobuf::Message* allocate_document_buffer(size_t);
    void deallocate_document_buffer(google::protobuf::Message*);
    google::protobuf::Message* get_nth_document(
            google::protobuf::Message* docs, size_t n);
    void* read(google::protobuf::Message&);
    void* sample(void*);
    void* sort(void*);
    void* sample_eta(void*);
    void* predict(void*);
    void* update(void*);
    void* optimize(void*);
    void* eval(void*, double&);
    void write(void*);
    void iteration_done();

    void* test(void*);
    void print_result();

    static long doc_index; //Running count of all the documents processed by the optimizer

private:
    void set_up_io(string, string, string);
    void release_io();

protected:
    TypeTopicCounts& _ttc;
    Parameter& _alpha;
    Parameter& _beta;
    Parameter* _eta;
    bool ignore_old_topic;
    int _num_words, _num_topics, _num_words_train, _num_label;
    //Reader
    DocumentReader *_wdoc_rdr, *_tdoc_rdr;;
    DocumentWriter *_tdoc_writer;

    //Train topic count table
    TypeTopicCounts *_ttc_train;

    //boost RNG
    base_generator_type *generators[NUM_RNGS];
    uniform_real<> *uni_dists[NUM_RNGS];
    variate_generator<base_generator_type&, boost::uniform_real<> >
            *unif01[NUM_RNGS];
    long rng_ind;

    //Useful variables
    double initial_c, nusqinv, delta_ell;
    size_t _num_docs;
    atomic<int> num_accurate;

    double **phi;

    double gibbs_converged;
    int gibbs_max_iter;
    int sampling_lag;

    tbb::mutex mut;
    vector< vector<label_score_t> > pred_list;
};

}
#endif /* MEDLDA_MODEL_TESTER_H_ */
