/*******************************************************************************
    Copyright (c) 2011 Yahoo! Inc. All rights reserved.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License. See accompanying LICENSE file.

    The Initial Developer of the Original Code is Shravan Narayanamurthy.
******************************************************************************/
/*
 * Simple_Execution_Strategy.cpp
 *
 *  Created on: 28-Dec-2010
 *      
 */

#include "Testing_Execution_Strategy.h"
#include "Context.h"
#include "Pipeline.h"
#include "tbb/tick_count.h"
#include "glog/logging.h"

Testing_Execution_Strategy::Testing_Execution_Strategy(Pipeline& pipeline,
        Model& model) :
    _pipeline(pipeline), _model(model) {
}

Testing_Execution_Strategy::~Testing_Execution_Strategy() {
}

void Testing_Execution_Strategy::execute() {
    Context& context = Context::get_instance();
    int loglikelihood_interval = context.get_int("printloglikelihood");

    int end_iter = context.get_int("iter");
    bool enable_likelihood = context.get_int("enable_likelihood");

    //LOG(WARNING) << "Starting Parallel testing Pipeline";
    tbb::tick_count start = tbb::tick_count::now();
    for (int iter = 0; iter <= 0; ++iter) {
    //while (true) {
        _pipeline.init();
        _pipeline.add_reader();
        _pipeline.add_tester();
        _pipeline.add_writer();

        tbb::tick_count t0 = tbb::tick_count::now();
        _pipeline.run();
        tbb::tick_count t1 = tbb::tick_count::now();
        //LOG(WARNING) << "Iteration " << iter << " done";
        //        << (t1 - t0).seconds() / 60 << " mins" << endl;
        _pipeline.clear();
        _pipeline.destroy();
    }
    //LOG(WARNING) << "Parallel testing Pipeline done";
    tbb::tick_count end = tbb::tick_count::now();
    LOG(WARNING) << "============= Test time: " << (end - start).seconds() << endl;
}
