/*******************************************************************************
    Copyright (c) 2011 Yahoo! Inc. All rights reserved.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License. See accompanying LICENSE file.

    The Initial Developer of the Original Code is Shravan Narayanamurthy.
******************************************************************************/
/*
 * Simple_Execution_Strategy.cpp
 *
 *  Created on: 28-Dec-2010
 *      
 */
#include <mpi.h>

#include "Training_Execution_Strategy.h"
#include "Context.h"
#include "tbb/tick_count.h"
#include "glog/logging.h"

#include "unistd.h"

Training_Execution_Strategy::Training_Execution_Strategy(Pipeline& pipeline,
        Model& model, Checkpointer& checkpointer) :
    _pipeline(pipeline), _model(model), _checkpointer(checkpointer) {
}

Training_Execution_Strategy::~Training_Execution_Strategy() {
}

void Training_Execution_Strategy::execute() {
    Context& context = Context::get_instance();
    int end_iter = context.get_int("iter");

    //MPI
    char processor_name[MPI_MAX_PROCESSOR_NAME];
    int name_len;
    MPI_Get_processor_name(processor_name, &name_len);
    int world_rank;
    MPI_Comm_rank(MPI_COMM_WORLD, &world_rank);
    stringstream ss;
    ss << "At " << processor_name << " (rank " << world_rank << "). ";
    string nodeinfo = ss.str();

    int MaxSamples = 1;

    //LOG(WARNING) << "Starting Parallel training Pipeline";
    tbb::tick_count start = tbb::tick_count::now();
    for (int iter = 1; iter <= end_iter; ++iter) {
        _pipeline.init();
        _pipeline.add_reader();
        _pipeline.add_sampler();
        _pipeline.add_updater();
        _pipeline.add_eta_sampler();
        _pipeline.add_writer();

        tbb::tick_count t0 = tbb::tick_count::now();
        _pipeline.run();
        //usleep(500000);
        tbb::tick_count t1 = tbb::tick_count::now();
        LOG(WARNING) <<  nodeinfo << "Iteration " 
                << iter << " (seconds): " << (t1 - t0).seconds() << endl;
        _pipeline.clear();
        _pipeline.destroy();
    }
    //LOG(WARNING) << nodeinfo << "==== Training time: " << train_time << endl;
    for (int iter = 1; iter <= MaxSamples; ++iter) {
        LOG(WARNING) << nodeinfo << "draw samples...";
        _pipeline.init();
        _pipeline.add_reader();
        _pipeline.add_sampler();
        _pipeline.add_updater();
        _pipeline.add_eta_sampler();
        //_pipeline.add_predictor();
        _pipeline.add_writer();
        _pipeline.run();
        //usleep(500000);
        _pipeline.clear();
        _pipeline.destroy();
    }
    tbb::tick_count end = tbb::tick_count::now();
    LOG(WARNING) << nodeinfo << "============= Training time: " << (end - start).seconds() << endl;
}
