// 计算 instance-based recall, precision, accuracy
// accuracy 按照 每个 instance 正确 算一分 错误算负一分 最低零分

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string>
#include <vector>
#include "gflags/gflags.h"
#include "glog/logging.h"
#include <fstream>

using namespace std;

DEFINE_string(pred_label_file, "", "");
DEFINE_string(gnd_label_file, "", "");

void ReadFile(std::string file_name, vector<vector<int> > *labels) {
  ifstream fin(file_name.c_str());
  if (!fin) {
    CHECK(false) << "open " << file_name << " failed";
  }
  std::string buffer_line;
  while (getline(fin, buffer_line)) {
    vector<int> this_labels;
    size_t start_pos = 0;
    while (1) {
      size_t blank_pos = buffer_line.find(",", start_pos);
      if (blank_pos == std::string::npos) {
        this_labels.push_back(atoi(&buffer_line[start_pos]));
        break;
      }
      this_labels.push_back(atoi(&buffer_line[start_pos]));
      start_pos = blank_pos + 1;
    }
    labels->push_back(this_labels);
  }
}

void PrintLabel(const vector<vector<int> > labels) {
  for (size_t i = 0; i < labels.size(); ++i) {
    printf("%d", labels[i][0]);
    for (size_t j = 1; j < labels[i].size(); ++j) {
      printf(" %d", labels[i][j]);
    }
    printf("\n");
  }
}

void CalcRecall(const vector<vector<int> >&pred_labels, const vector<vector<int> >&gnd_labels) {
  CHECK(pred_labels.size() == gnd_labels.size());
  int64_t total_labels = 0;
  int64_t recall_labels = 0;
  for (size_t i = 0; i < gnd_labels.size(); ++i) {
    for (size_t j = 0; j < gnd_labels[i].size(); ++j) {
      ++total_labels;
      // check if gnd label in pred label
      int match = 0;
      for (size_t m = 0; m < pred_labels[i].size(); ++m) {
        if (gnd_labels[i][j] == pred_labels[i][m]) {
          match = 1;
          break;
        }
      }
      recall_labels += match;
    }
  }
  LOG(INFO) << "total_gnd_labels: " << total_labels << " recall_labels: " 
            << recall_labels << " recall: " << (double)recall_labels / total_labels;
}

void CalcPrecision(const vector<vector<int> >&pred_labels, const vector<vector<int> >&gnd_labels) {
  CHECK(pred_labels.size() == gnd_labels.size());
  int64_t total_labels = 0;
  int64_t precision_labels = 0;
  for (size_t i = 0; i < pred_labels.size(); ++i) {
    for (size_t j = 0; j < pred_labels[i].size(); ++j) {
      ++total_labels;
      // check if gnd label in pred label
      int match = 0;
      for (size_t m = 0; m < gnd_labels[i].size(); ++m) {
        if (pred_labels[i][j] == gnd_labels[i][m]) {
          match = 1;
          break;
        }
      }
      precision_labels += match;
    }
  }
  LOG(INFO) << "total_precision_labels: " << total_labels << " precision_labels: " 
            << precision_labels << " precision: " << (double)precision_labels / total_labels;
}

void CalcAccuray(const vector<vector<int> >&pred_labels, const vector<vector<int> >&gnd_labels) {
  CHECK(pred_labels.size() == gnd_labels.size());
  double total_accuracy = 0.0;
  
  for (size_t i = 0; i < pred_labels.size(); ++i) {
    int score = 0;
    for (size_t j = 0; j < pred_labels[i].size(); ++j) {
      // check if gnd label in pred label
      int match = -1;
      for (size_t m = 0; m < gnd_labels[i].size(); ++m) {
        if (pred_labels[i][j] == gnd_labels[i][m]) {
          match = 1;
          break;
        }
      }
      score += match;
    }
    if (score < 0) { score = 0; }
    CHECK(score <= gnd_labels[i].size());
    total_accuracy += (double)score / gnd_labels[i].size();
  }
  LOG(INFO) << "total_accuracy: " << total_accuracy / pred_labels.size();

}

int main(int argc, char *argv[]) {
  google::ParseCommandLineFlags(&argc, &argv, false);
  vector<vector<int> > pred_labels, gnd_labels;
  ReadFile(FLAGS_pred_label_file, &pred_labels);
  ReadFile(FLAGS_gnd_label_file, &gnd_labels);
  CalcRecall(pred_labels, gnd_labels);
  CalcPrecision(pred_labels, gnd_labels);
}

